package com.medcosm.pictureframe;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import java.util.GregorianCalendar;

import java.io.*;

import java.awt.image.*;
import java.awt.font.TextLayout;
import java.text.DateFormat;
import java.util.Date;
import java.awt.font.*;
import java.awt.geom.*;
import javax.imageio.*;

import java.util.Iterator;
import java.util.Locale;
import javax.imageio.stream.*;


/**
 * The actual full screen frame which images are loaded into.
 * <p>Title: MedCosm Picture Frame</p>
 * <p>Description:Short  Minimal Electronic Picture Frame Software</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: MedCosm</p>
 * @author Alan Stein
 * @version 1.0
 */

public class PictureFrame extends JFrame
{
  boolean pauseMode=false;
  String oldFileName=null;
  Thread parentThread=null;

  /**
   * Display in window, let user choose aspect (resize window)
   */
  public static final int MODE_WINDOWED_RESCALED=1;
  /**
   * Display in window, window size = actual image size
   */
  public static final int MODE_WINDOWED_UNSCALED=2;
  /**
   * Ignore aspect ratio, display full-screen
   */
  public static final int MODE_FULLSCREEN=0;

  /**
   * Running on FAST machine
   */
  public static final int MODE_CPU_FAST=1;
  /**
   * Running on SLOW machine
   */
  public static final int MODE_CPU_SLOW=0;

  JPanel contentPane;
  BorderLayout borderLayout1 = new BorderLayout();
  JLabel jLabel1 = new JLabel();

  Image image;

  //Construct the frame
  public PictureFrame()
  {
    try
    {
      jbInit();
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  public void setParentThread(Thread t)
  {
    parentThread=t;
  }

  //Component initialization
  private void jbInit() throws Exception
  {
    contentPane = (JPanel) this.getContentPane();
    jLabel1.setText("Image Should Load Momentarily...");
    contentPane.setLayout(borderLayout1);
    this.setSize(new Dimension(400, 300));
    this.setTitle("MedCosm PictureFrame");
    this.addKeyListener(new java.awt.event.KeyAdapter()
    {
      public void keyPressed(KeyEvent e)
      {
        this_keyPressed(e);
      }
    });
    contentPane.add(jLabel1, BorderLayout.CENTER);
  }

  /**
   * Loads an image and displays it on the screen
   * @param fileName file to load
   * @param date flag to superimpose date
   * @param aspectMode see class fields
   * @param cpuMode see class fields.  Will skip smoothing, antialiasing, etc.
   * @param saveImage flag to control writing of image dated image to same directory (with "_dated" appended to filename)
   * @param saveDir directory into which saved images will be written
   * @param saveImgQuality JPG compression parameter (0=high compression, 1=high quality)
   */
  public void loadImageFile(String fileName, boolean date, int aspectMode, int cpuMode,
      boolean saveImage, String saveDir, float saveImgQuality, int saveImgSize)
  {
    if(pauseMode)
      fileName=oldFileName;
    else
      oldFileName=fileName;

    System.out.print("Loading File:  "+fileName+"\n");
    BufferedImage biRaw=null; //buffered image directly from file
    BufferedImage biScaled=null; //buffered image scaled appropriately for screen

//    Toolkit kit = Toolkit.getDefaultToolkit();
//    image = kit.getImage(fileName);
//    bi=new BufferedImage(jLabel1.getWidth(),jLabel1.getHeight(), BufferedImage.TYPE_INT_RGB);
//    Graphics2D big2d = bi.createGraphics(); //so create graphic on bufIm
//    big2d.drawImage(image.getScaledInstance(jLabel1.getWidth(),jLabel1.getHeight(),Image.SCALE_FAST),0, 0,null);     //and redraw the Image as a BufImage

      File file=new File(fileName);   //get file
      long lDate=file.lastModified();
      java.util.Date d=new java.util.Date(lDate);
      GregorianCalendar gc=new GregorianCalendar();
      gc.setTime(d);    //and set GC

      String dateStr=gc.get(GregorianCalendar.DATE)+"."
          +(gc.get(GregorianCalendar.MONTH)+1)+"."  //construct date string
          +gc.get(GregorianCalendar.YEAR);

    try
    {
      biRaw = ImageIO.read(file); //read file into biRaw
    }
    catch (IOException ex)
    {
      System.out.print("IOError loading "+fileName+"\n");
      return;
    }

    if(biRaw==null)
    {
      System.out.print("InvalidFileError loading "+fileName+"\n");
      return;
    }

    Graphics2D g=null;

    if(saveImage)
    {
      if(saveImgSize!=FrameJPGquality.SIZE_ORIG)
      {
        int x=0, y=0;
        switch (saveImgSize)
        {
          case FrameJPGquality.SIZE_1024:
            x=1024;
            y=768;
            break;
          case FrameJPGquality.SIZE_1280:
            x=1280;
            y=1024;
            break;
          case FrameJPGquality.SIZE_640:
            x=640;
            y=480;
            break;
          case FrameJPGquality.SIZE_800:
            x=800;
            y=600;
            break;
        }

        biScaled=new BufferedImage(x, y, BufferedImage.TYPE_INT_RGB);
        g = biScaled.createGraphics(); //so create graphic on bufIm

        if(cpuMode==MODE_CPU_FAST)
          g.drawImage(biRaw.getScaledInstance(x,y,Image.SCALE_SMOOTH),0, 0,null);     //and redraw the Image as a BufImage
        else
          g.drawImage(biRaw.getScaledInstance(x,y,Image.SCALE_FAST),0, 0,null);     //and redraw the Image as a BufImage
      }
      else
        biScaled=biRaw;

    }
    else
    if(aspectMode==MODE_FULLSCREEN)
    {
      //and create scaled image into biScaled
      biScaled=new BufferedImage(jLabel1.getWidth(),jLabel1.getHeight(), BufferedImage.TYPE_INT_RGB);
      g = biScaled.createGraphics(); //so create graphic on bufIm

      if(cpuMode==MODE_CPU_FAST)
        g.drawImage(biRaw.getScaledInstance(jLabel1.getWidth(),jLabel1.getHeight(),Image.SCALE_SMOOTH),0, 0,null);     //and redraw the Image as a BufImage
      else
        g.drawImage(biRaw.getScaledInstance(jLabel1.getWidth(),jLabel1.getHeight(),Image.SCALE_FAST),0, 0,null);     //and redraw the Image as a BufImage
    }
    else
    if(aspectMode==MODE_WINDOWED_UNSCALED)
    {
      biScaled=new BufferedImage(jLabel1.getWidth(),jLabel1.getHeight(), BufferedImage.TYPE_INT_RGB);
    }

    if(date)  //sample code for watermarking from http://forum.java.sun.com/thread.jsp?forum=20&thread=505366&tstart=75&trange=15
    {
      g = biScaled.createGraphics(); //create graphic on bufIm
      if(cpuMode==MODE_CPU_FAST)
      {
        g.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
        g.setRenderingHint(RenderingHints.KEY_FRACTIONALMETRICS, RenderingHints.VALUE_FRACTIONALMETRICS_ON);
        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
      }
      else
      {
        g.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);
        g.setRenderingHint(RenderingHints.KEY_FRACTIONALMETRICS, RenderingHints.VALUE_FRACTIONALMETRICS_OFF);
        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);
      }

      g.setFont(new Font("Lucida Bright", Font.ITALIC, 60));
//      g.rotate(-Math.PI/4, biScaled.getWidth()/2, biScaled.getHeight()/2);  //rotate text if desired
      TextLayout tl = new TextLayout(dateStr, g.getFont(), g.getFontRenderContext());
      Rectangle2D bounds = tl.getBounds();
//      double x = (biScaled.getWidth()-bounds.getWidth())/2 - bounds.getX();     //center text
//      double y = (biScaled.getHeight()-bounds.getHeight())/2 - bounds.getY();
      double x = biScaled.getWidth()-bounds.getWidth() - bounds.getX();   //place text in lower right corner
      double y = biScaled.getHeight()-bounds.getHeight() - bounds.getY();
//      System.out.print("rawWidth="+biRaw.getWidth()+" scaledWidth="+biScaled.getWidth()+
//          " boundsWidth="+bounds.getWidth()+" boundsX="+bounds.getX()+
//          " x="+x+" y="+y+"\n");
      Shape outline = tl.getOutline(AffineTransform.getTranslateInstance(x+2, y+1));
      g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.6f));
      g.setPaint(Color.WHITE);
      g.draw(outline);
      g.setPaint(new GradientPaint(0, 0, Color.ORANGE, 30, 20, new Color(128,128,255), true));
      tl.draw(g, (float)x, (float)y);
      g.dispose();
    }


    if(pauseMode)  //sample code for watermarking from http://forum.java.sun.com/thread.jsp?forum=20&thread=505366&tstart=75&trange=15
    {
//      System.out.print("Pause Mode\n");
      g = biScaled.createGraphics(); //create graphic on bufIm
      g.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);
      g.setRenderingHint(RenderingHints.KEY_FRACTIONALMETRICS, RenderingHints.VALUE_FRACTIONALMETRICS_OFF);
      g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);

      g.setFont(new Font("Lucida Bright", Font.ITALIC, 60));
      TextLayout tl = new TextLayout("Paused", g.getFont(), g.getFontRenderContext());
      Rectangle2D bounds = tl.getBounds();
      double x = 0;   //place text in lower right corner
      double y = biScaled.getHeight()-bounds.getHeight() - bounds.getY();
//      System.out.print("rawWidth="+biRaw.getWidth()+" scaledWidth="+biScaled.getWidth()+
//          " boundsWidth="+bounds.getWidth()+" boundsX="+bounds.getX()+
//          " x="+x+" y="+y+"\n");
      Shape outline = tl.getOutline(AffineTransform.getTranslateInstance(x+2, y+1));
      g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.8f));
      g.setPaint(Color.WHITE);
      g.draw(outline);
      g.setPaint(new GradientPaint(0, 0, Color.ORANGE, 30, 20, new Color(128,128,255), true));
      tl.draw(g, (float)x, (float)y);
      g.dispose();
    }

    if(saveImage)
    {
      String shortFileName=file.getName().substring(0,file.getName().lastIndexOf("."))+"_cpy.jpg";
//      String fOutName=file.getParent()+File.separator+shortFileName;
      String fOutName=saveDir+File.separator+shortFileName;
      System.out.print("Saving file="+fOutName+"\n");

      try
      {
//        ImageIO.write(biScaled, "jpeg", new File(fOutName));  //single line to write JPG, but can't control quality using this method

        //or the following set of lines which allows more control over JPEG written
        // Find a jpeg writer
        ImageWriter writer = null;
        Iterator iter = ImageIO.getImageWritersByFormatName("jpg");
        if (iter.hasNext())
        {
          writer = (ImageWriter) iter.next();
        }

        // Prepare output file
        ImageOutputStream ios = ImageIO.createImageOutputStream(new File(fOutName));
        writer.setOutput(ios);

        // Set the compression quality
        ImageWriteParam iwparam = new javax.imageio.plugins.jpeg.JPEGImageWriteParam(Locale.getDefault());
        iwparam.setCompressionMode(ImageWriteParam.MODE_EXPLICIT);
//        iwparam.setCompressionQuality(0.85f);
        iwparam.setCompressionQuality((float)saveImgQuality);

        // Write the image
        writer.write(null, new IIOImage(biScaled, null, null), iwparam);

        // Cleanup
        ios.flush();
        writer.dispose();
        ios.close();
      }
      catch (IOException ex1)
      {
        ex1.printStackTrace();
      }
    }

    //last thing we do is set Image to component
    jLabel1.setIcon(new ImageIcon(biScaled));
  }


  public boolean getPauseMode()
  {
    return pauseMode;
  }


  /**
   * user has pressed a key...  kill the pictureframe, or at least toggle pause and wakeup the sleeping thread
   * @param e
   */
  void this_keyPressed(KeyEvent e)
  {
    parentThread.interrupt();   //wake up sleeping thread

    if((e.getKeyChar()== java.awt.Event.ESCAPE) || (e.getKeyChar()=='q') || (e.getKeyChar()=='Q'))
      this.dispose();
    else
    if(e.getKeyChar()==' ')
      pauseMode = !pauseMode;
  }



//  and here's some code for watermarking with an image instead of text
//
//  public class WaterMark {
//      public static void main(String[] args) throws IOException {
//          URL url = new URL("http://today.java.net/jag/bio/JagHeadshot-small.jpg");
//          BufferedImage im = ImageIO.read(url);
//          URL url2 = new URL("http://weblogs.java.net/jag/Image54-large.jpeg");
//          BufferedImage im2 = ImageIO.read(url2);
//          Graphics2D g = im.createGraphics();
//          g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.4f));
//          g.drawImage(im2, (im.getWidth()-im2.getWidth())/2, (im.getHeight()-im2.getHeight())/2, null);
//          g.dispose();
//
//          display(im);
//          ImageIO.write(im, "jpeg", new File("sample_output.jpeg"));
//      }
//
//      public static void display(BufferedImage image) {
//          JFrame f = new JFrame("WaterMark");
//          f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
//          f.getContentPane().add(new JLabel(new ImageIcon(image)));
//          f.pack();
//          f.setLocationRelativeTo(null);
//          f.setVisible(true);
//      }
//  }




}